﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using OpenTK;
using OpenTK.Graphics;
using OpenTK.Graphics.OpenGL;

using CG1.Ex03.Meshes;
using CG1.Ex03.Rendering;

namespace CG1.Ex03
{
    public partial class Form1 : Form
    {
        #region Properties

        //Boolean using System, not OpenTK
        System.Boolean Loaded = false;
        World OurWorld;
        Point LastPos;
        ViewMode Mode;

        //Mesh
        Mesh OurMainMesh;

        #endregion

        #region OpenGL GUI

        /// <summary>
        /// OpenGL Initialization. Also a good place to set viewport and projection. Load resources here.
        /// </summary>
        private void glControl1_Load(object sender, EventArgs e)
        {
            Loaded = true;

            //Load our meshes
            OurMainMesh = new Mesh("Input/Crate.obj", "Input/CrateDefault.png");
            Mode = ViewMode.PerspectiveScene;

            //Init our world
            OurWorld = new World(glControl1.Width, glControl1.Height);
            OurWorld.InitGL();

            glControl1.Invalidate();
        }

        /// <summary>
        /// Called when your window is resized. Set your viewport here. It is also
        /// a good place to set up your projection matrix (which probably changes
        /// along when the aspect ratio of your window).
        /// </summary>
        private void glControl1_Resize(object sender, EventArgs e)
        {
            //Play nice
            if (!Loaded)
                return;

            OurWorld.ReshapeGL(glControl1.Width, glControl1.Height);
            //InitGL();
            glControl1.Invalidate();
        }

        /// <summary>
        /// Called when it is time to render the next frame. Add your rendering code here.
        /// </summary>
        private void glControl1_Paint(object sender, PaintEventArgs e)
        {
            //Play nice
            if (!Loaded)
                return;

            OurWorld.Render();
            OurMainMesh.Render();

            //double buffered graphics
            glControl1.SwapBuffers();
        }

        /// <summary>
        /// Called when it is time to setup the next frame. Add you game logic here.
        /// </summary>
        private void glControl1_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.F1:
                    Mode = ViewMode.OrthogonalScene;
                    OurWorld.Viewport(Mode);
                    ModeView.Text = "Mode: Orthogonal";
                    break;
                case Keys.F2:
                    Mode = ViewMode.PerspectiveScene;
                    OurWorld.Viewport(Mode);
                    ModeView.Text = "Mode: Perspective";
                    break;
                case Keys.Escape:
                    Close();
                    break;
            }
            glControl1.Invalidate();
        }

        /// <summary>
        /// Mouse movement. Add rotation angles of camera and other changing properties here.
        /// </summary>
        private void glControl1_MouseMove(object sender, MouseEventArgs e)
        {
            Int32 dx = e.X - LastPos.X;
            Int32 dy = e.Y - LastPos.Y;
            if (e.Button == MouseButtons.Left)
            {
                OurWorld.CameraFi += dx * 0.3;
                OurWorld.CameraTheta += dy * 0.3;
            }
            LastPos = e.Location;
            glControl1.Invalidate();
        }

        /// <summary>
        /// Manually added mouse wheel handle. Use e.g. to change distance from objects.
        /// </summary>
        private void glControl1_MouseWheel(object sender, MouseEventArgs e)
        {
            switch (Mode)
            {
                case ViewMode.OrthogonalScene:
                    OurWorld.OrtCamDis += e.Delta * 0.0005f;
                    OurWorld.Viewport(Mode);
                    break;
                case ViewMode.PerspectiveScene:
                    OurWorld.PerCamDis += e.Delta * 0.01f;
                    break;
                default:
                    GL.LoadIdentity();
                    break;
            }
            glControl1.Invalidate();
        }

        #endregion

        #region Application GUI

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Load file
        /// </summary>
        private void button1_Click(object sender, EventArgs e)
        {
            try
            {
                ReadDialog();
            }
            catch (Exception)
            {
                MessageBox.Show("Error during loading files. Select only two files - first one *.obj and a second one *.png.");
                return;
            }

            glControl1.Invalidate();
        }

        /// <summary>
        /// Very simple loading. Improve if needed.
        /// How to: Select two files - *.obj and *.png.
        /// First select *.obj and than *.png
        /// </summary>
        private void ReadDialog()
        {
            OpenFileDialog OpenFile = new OpenFileDialog();
            OpenFile.CheckFileExists = true;
            OpenFile.AddExtension = true;
            OpenFile.Multiselect = true;
            OpenFile.Filter = "All files (*.*)|*.*";

            if (OpenFile.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                String TexFile, ObjFile;
                ObjFile = OpenFile.FileNames[1].EndsWith("obj") ? OpenFile.FileNames[1] : OpenFile.FileNames[0];
                TexFile = OpenFile.FileNames[1].EndsWith("png") ? OpenFile.FileNames[1] : OpenFile.FileNames[0];
                OurMainMesh = new Mesh(ObjFile, TexFile);
                GL.ClearColor(Color.FromArgb(57, 57, 57));
            }
        }

        #endregion
    }
}
